(* aio.mli: Linux async I/O interface for libaio-ocaml
 * Copyright (C) 2009 Goswin von Brederlow
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * Under Debian a copy can be found in /usr/share/common-licenses/LGPL-3.
 *)

(** libaio-ocaml Linux async I/O interface for ocaml

    This module implements the libaio bindings that interface with the
    Linux system calls.

    {e Version 0.0.0 - goswin-v-b\@web.de }

    {1:top  }
*)

type buffer
  (** The type for Buffer. *)

val buffer : int -> buffer
  (** Allocate an uninitialized buffer. *)

val get_string : buffer -> int -> string
  (** Get a string of length x from a buffer. *)

val put_string : buffer -> string -> unit
  (** Put a string into a buffer. *)

val rewind : buffer -> unit
  (** Rewind a buffer to it start. *)

type result
  (** The type for a result of a completed I/O request *)

exception Error of int
  (** An error has occured during a request. *)

exception Incomplete of buffer * int
  (** A request was only partialy completed. *)

val result : result -> buffer
  (** Extract the buffer from a result or throw the proper exception *)

type context
  (** The type for a libaio Context. *)

val context : int -> context
  (** Create a new context for n simultaneous requests. *)

val read : context -> Unix.file_descr -> buffer -> int64 -> (result -> unit) -> unit
  (** fill buffer from file at given offset and call continuation *)

val write : context -> Unix.file_descr -> buffer -> int64 -> (result -> unit) -> unit
  (** write buffer to file at given offset and call continuation *)

val run : context -> unit
  (** run the context till there are no more pending requests *)
